@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.74 2005/09/15 16:45:27 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.75 2005/11/15 18:02:06 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -421,7 +421,7 @@ void do_numtype(enum num_type);
 void do_expno();
 void do_immed();
 void do_memory();
-void do_mode();
+void do_mode(int);
 void do_none();
 void do_number();
 void do_paren();
