@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.70 2005/07/15 15:10:32 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.73 2005/11/15 18:02:06 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -416,8 +416,19 @@ do_expression()
 	        v->e.calc_complete = 1;
 		goto out;
             } else {
-	        update_statusbar(_("Malformed expression"), 
-                                 "gtk-dialog-error");
+		char *message = NULL;
+		switch (ret) {
+		case -PARSER_ERR_INVALID_BASE:
+		    message = _("Invalid number for the current base");
+		    break;
+		case -PARSER_ERR_TOO_LONG_NUMBER:
+		    message = _("Too long number");
+		    break;
+		default:
+		    message = _("Malformed expression");
+		    break;
+		}
+	        update_statusbar(message, "gtk-dialog-error");
 	        return;
             }
         } else {
@@ -443,7 +454,7 @@ void
 do_calc()      /* Perform arithmetic calculation and display result. */
 {
     double dval, dres;
-    int MP1[MP_SIZE];
+    int MP1[MP_SIZE], MP2[MP_SIZE];
 
     if (!(v->opsptr && !v->show_paren)) {  /* Don't do if processing parens. */
         if (key_equal(v->current, KEY_EQ) && 
@@ -480,6 +491,18 @@ do_calc()      /* Perform arithmetic calculation and display result. */
     } else if (IS_KEY(v->cur_op, KEY_DIV.value[0])) {     /* Division. */
         mpdiv(v->MPresult, v->MPdisp_val, v->MPresult);
 
+    } else if (IS_KEY(v->cur_op, KEY_MOD.value[0])) {     /* Modulus. */
+        mpcmim(v->MPresult, MP1);
+        mpcmim(v->MPdisp_val, MP2);
+        if (!mpeq(v->MPresult, MP1) || !mpeq(v->MPdisp_val, MP2)) {
+            doerr(_("Error, operands must be integers"));
+        }
+
+        mpdiv(v->MPresult, v->MPdisp_val, MP1);
+        mpcmim(MP1, MP1);
+        mpmul(MP1, v->MPdisp_val, MP2);
+        mpsub(v->MPresult, MP2, v->MPresult);
+
     } else if (IS_KEY(v->cur_op, KEY_PER.value[0])) {     /* % */
         mpmul(v->MPresult, v->MPdisp_val, v->MPresult);
         MPstr_to_num("0.01", DEC, MP1);
@@ -928,7 +951,7 @@ do_memory()
 
 
 void
-do_mode()                   /* Set special calculator mode. */
+do_mode(int toclear)           /* Set special calculator mode. */
 {
     char title[MAXLINE];
 
@@ -936,7 +959,9 @@ do_mode()                   /* Set special calculator mode. */
     set_title(FCP_KEY, title);
     put_resource(R_MODE, Rmstr[(int) v->modetype]);
     set_mode(v->modetype);
-    do_clear();
+    if (toclear) {
+        do_clear();
+    }
 }
 
 
